@@ -167,6 +167,7 @@ pub fn eval_cyc_from_str(
     out_mode: OutputMode,
     template_events: &[String],
     event_mappings: &HashMap<String, Vec<SourceEvent>>,
+    global_parameters: &sync::Arc<GlobalParameters>,
 ) -> Vec<Vec<CycleResult>> {
     let items = parse_cyc(src.trim()).map_err(|e: nom::Err<VerboseError<&str>>| {
         let ret = format!("{:#?}", e);
@@ -221,8 +222,14 @@ pub fn eval_cyc_from_str(
                             match parse_expr(name.trim()) {
                                 Ok((_, expr)) => {
                                     if let Some(Expr::Constant(Atom::SoundEvent(e))) =
-                                        eval_expression(expr, sample_set, out_mode)
+                                        eval_expression(
+                                            expr,
+                                            sample_set,
+                                            out_mode,
+                                            global_parameters,
+                                        )
                                     {
+                                        //println!("ev {}", e.name);
                                         cycle_position.push(CycleResult::SoundEvent(e));
                                     } else {
                                         println!("couldn't eval cycle expr");
@@ -286,7 +293,7 @@ pub fn eval_cyc_from_str(
                         match parse_expr(ev_name.trim()) {
                             Ok((_, expr)) => {
                                 if let Some(Expr::Constant(Atom::SoundEvent(e))) =
-                                    eval_expression(expr, sample_set, out_mode)
+                                    eval_expression(expr, sample_set, out_mode, global_parameters)
                                 {
                                     cycle_position.push(CycleResult::SoundEvent(e));
                                 } else {
@@ -299,6 +306,7 @@ pub fn eval_cyc_from_str(
                         }
                     }
                 }
+                //println!("r {} p {}", results.len(), cycle_position.len());
                 results.push(cycle_position);
             }
             results
